# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from six.moves.urllib import parse

from bs4 import BeautifulSoup
from requests import RequestException, TooManyRedirects

from school_api.client.api.base import BaseSchoolApi
from school_api.client.api.utils import get_alert_tip, get_view_state_from_html
from school_api.client.api.utils.schedule_parse import ScheduleParse
from school_api.client.utils import ScheduleType, UserType
from school_api.utils import to_text
from school_api.exceptions import ScheduleException


class Schedule(BaseSchoolApi):
    schedule_type = None
    schedule_year = None
    schedule_term = None
    schedule_url = None

    def get_schedule(self, schedule_year=None, schedule_term=None, schedule_type=None, **kwargs):
        ''' 课表信息 获取入口
        :param schedule_year: 课表学年
        :param schedule_term: 课表学期
        :param schedule_type: 0.个人课表 1.班级课表
        :param kwargs: requests模块参数
        :return:
        '''
        self.schedule_type = ScheduleType.CLASS if self.user.user_type \
            else schedule_type or ScheduleType.PERSON
        self.schedule_year = schedule_year
        self.schedule_term = str(schedule_term) if schedule_term else schedule_term
        self.schedule_url = self.school_url["SCHEDULE_URL"][self.schedule_type]

        if self.user.user_type != UserType.DEPT:
            self.schedule_url += self.user.account
            data = self._get_api(**kwargs)
        else:
            self.schedule_url += parse.quote(self.user.account.encode('gb2312'))
            data = self._get_api_by_bm(**kwargs)
        if self.schedule_term and self.schedule_year and (
                self.schedule_term != data["schedule_term"] or self.schedule_year != data["schedule_year"]):
            raise ScheduleException(self.code, '暂无课表信息')
        return data

    def _get_api(self, **kwargs):
        coding = 'gbk' if self.user.user_type else 'GB18030'
        try:
            res = self._get(self.schedule_url, **kwargs)
        except TooManyRedirects:
            raise ScheduleException(self.code, '课表接口已关闭')
        except RequestException:
            raise ScheduleException(self.code, '获取课表请求参数失败')

        html = res.content.decode(coding)
        tip = get_alert_tip(html)
        if tip:
            raise ScheduleException(self.code, tip)

        schedule = ScheduleParse(html, self.time_list, self.schedule_type).get_schedule_dict()
        # 第一次请求的时候，教务系统默认返回当前学年学期课表
        # 如果设置了学年跟学期，则获取指定学年学期的课表
        if self.schedule_year and self.schedule_term and (
                self.schedule_year != schedule['schedule_year'] or self.schedule_term != schedule['schedule_term']):

            payload = self._get_payload(res.text)

            try:
                res = self._post(self.schedule_url, data=payload, **kwargs)
            except RequestException:
                raise ScheduleException(self.code, '获取课表信息失败')

            html = res.content.decode(coding)
            schedule = ScheduleParse(
                html,
                self.time_list,
                self.schedule_type
            ).get_schedule_dict()

        return schedule

    def _get_api_by_bm(self, class_name, **kwargs):
        ''' 部门教师 查询学生班级课表 共3个请求'''

        # steps 1: 获取课表页面 参数信息
        try:
            res = self._get(self.schedule_url, **kwargs)
        except RequestException:
            raise ScheduleException(self.code, '获取课表请求参数失败')

        # steps 2: 选择课表 学年学期
        if self.schedule_year and self.schedule_term:
            payload = self._get_payload(res.text)
            try:
                res = self._post(self.schedule_url, data=payload, **kwargs)
            except RequestException:
                raise ScheduleException(self.code, '获取课表请求参数失败')

        # steps 3: 获取课表数据
        payload = self._get_payload_by_bm(res.content.decode('gbk'), class_name)
        try:
            res = self._post(self.schedule_url, data=payload, **kwargs)
        except RequestException:
            raise ScheduleException(self.code, '获取课表信息失败')

        html = res.content.decode('gbk')
        schedule = ScheduleParse(html, self.time_list, self.schedule_type).get_schedule_dict()
        return schedule

    def _get_payload(self, html):
        ''' 获取课表post 的参数 '''
        view_state = get_view_state_from_html(html)
        payload = {
            '__VIEWSTATE': view_state,
            ['xnd', 'xn'][self.schedule_type]: self.schedule_year,
            ['xqd', 'xq'][self.schedule_type]: self.schedule_term
        }
        return payload

    def _get_payload_by_bm(self, html, class_name):
        ''' 提取页面参数用于请求课表 '''
        pre_soup = BeautifulSoup(html, "html.parser")
        view_state = pre_soup.find(attrs={"name": "__VIEWSTATE"})['value']
        schedule_id_list = pre_soup.find(id='kb').find_all('option')
        class_name = to_text(class_name)
        for name in schedule_id_list:
            if name.text == class_name:
                schedule_id = name['value']
                break
        else:
            raise ScheduleException(self.code, '暂无该班级课表信息')

        # 获取班级课表
        payload = {
            '__VIEWSTATE': view_state,
            'kb': schedule_id
        }
        return payload



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from bs4 import BeautifulSoup
from requests import RequestException, TooManyRedirects

from school_api.client.api.base import BaseSchoolApi
from school_api.client.api.utils import get_alert_tip
from school_api.exceptions import ScoreException


class Score(BaseSchoolApi):
    ''' 学生成绩获取 '''

    def get_score(self, score_year=None, score_term=None, use_api=0, **kwargs):
        ''' 成绩信息 获取入口
        :param score_year: 成绩学年
        :param score_term: 成绩学期
        :param use_api:    0.接口1, 1.接口2, 2.接口3 ...
        :param kwargs: requests模块参数
        return
        '''
        score_url = self.school_url['SCORE_URL'][use_api] + self.user.account

        try:
            view_state = self._get_view_state(score_url, **kwargs)
        except TooManyRedirects:
            msg = '可能是成绩接口地址不对，请尝试更改use_api值'
            raise ScoreException(self.code, msg)
        except RequestException:
            msg = '获取成绩请求参数失败'
            raise ScoreException(self.code, msg)

        payload = {
            '__VIEWSTATE': view_state,
            'Button2': '在校学习成绩查询',
            'btn_zcj': '历年成绩',
            'btnCx': ' 查  询 ',
            'ddlXN': '',
            'ddlXQ': ''
        }
        try:
            res = self._post(score_url, data=payload, **kwargs)
        except TooManyRedirects:
            raise ScoreException(self.code, '成绩接口已关闭')
        except RequestException:
            raise ScoreException(self.code, '获取成绩信息失败')

        html = res.content.decode('GB18030')
        tip = get_alert_tip(html)
        if tip:
            raise ScoreException(self.code, tip)

        return ScoreParse(self.code, html, use_api).get_score(score_year, score_term)


class ScoreParse():
    ''' 成绩页面解析模块 '''

    def __init__(self, code, html, use_api):
        self.code = code
        self.use_api = use_api
        self.soup = BeautifulSoup(html, "html.parser")
        self._html_parse_of_score()

    def _html_parse_of_score(self):
        tag = "Datagrid1" if self.use_api != 3 else "DataGrid1"
        table = self.soup.find("table", {"id": tag})
        if not table:
            raise ScoreException(self.code, '获取成绩信息失败')

        rows = table.find_all('tr')
        rows.pop(0)
        self.score_info = {}
        for row in rows:
            cells = row.find_all("td")
            # 学年学期
            year = cells[0].text
            term = cells[1].text
            # 课程名
            lesson_name = cells[3].text.strip()
            credit = cells[6].text.strip() or 0
            point = cells[7].text.strip() or 0
            score = cells[8].text.strip() or 0
            score_dict = {
                "lesson_name": lesson_name,
                "credit": float(credit),
                "point": float(point),
                "score": self.handle_data(score)
            }
            # 有其他成绩内容则输出
            makeup_score = cells[10].text
            retake_score = cells[11].text
            if makeup_score != '\xa0':
                # 补考成绩
                score_dict['bkcj'] = makeup_score
            if retake_score != '\xa0':
                # 重修成绩
                score_dict['cxcj'] = retake_score
            # 组装数组格式的数据备用
            self.score_info[year] = self.score_info.get(year, {})
            self.score_info[year][term] = self.score_info[year].get(term, [])
            self.score_info[year][term].append(score_dict)

    def get_score(self, year, term):
        ''' 返回成绩信息json格式 '''
        try:
            if not self.score_info:
                raise KeyError
            if year:
                if term:
                    return self.score_info[year][term]
                return self.score_info[year]
        except KeyError:
            raise ScoreException(self.code, '暂无成绩信息')

        return self.score_info

    @staticmethod
    def handle_data(data):
        try:
            return float(data)
        except ValueError:
            return data



# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals
import six


class ObjectDict(dict):
    """:copyright: (c) 2014 by messense.
    Makes a dictionary behave like an object, with attribute-style access.
    """

    def __getattr__(self, key):
        if key in self:
            return self[key]
        return None

    def __setattr__(self, key, value):
        self[key] = value

    def __getstate__(self):
        return None


def to_text(value, encoding='utf-8'):
    """:copyright: (c) 2014 by messense.
    Convert value to unicode, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return ''
    if isinstance(value, six.text_type):
        return value
    if isinstance(value, six.binary_type):
        return value.decode(encoding)
    return six.text_type(value)


def to_binary(value, encoding='utf-8'):
    """Convert value to binary string, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return b''
    if isinstance(value, six.binary_type):
        return value
    if isinstance(value, six.text_type):
        return value.encode(encoding)
    return to_text(value).encode(encoding)



#!/usr/bin/env python
# coding=utf-8

from setuptools import setup, find_packages

with open('README.md', 'rb') as f:
    long_description = f.read().decode('utf-8').split("## User permissions")[0]

setup(
    name="School-Api",
    author="dairoot",
    version="1.4.6",
    license='MIT',
    author_email="623815825@qq.com",
    description="School SDK for Python",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/dairoot/school-api',
    packages=find_packages(),
    package_data={'school_api': ['check_code/theta.dat'], },
    include_package_data=True,
    platforms='any',
    zip_safe=False,

    install_requires=[
        'six',
        'requests',
        'redis',
        'bs4',
        'pillow',
        'numpy'
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]

)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from bs4 import BeautifulSoup
from requests import RequestException, TooManyRedirects
from school_api.client.api.base import BaseSchoolApi
from school_api.exceptions import UserInfoException


class UserlInfo(BaseSchoolApi):
    ''' 用户信息查询 部门教师不可用 '''

    def get_info(self, **kwargs):
        ''' 用户信息 获取入口 '''
        info_url = self.school_url['INFO_URL'] + self.user.account

        try:
            res = self._get(info_url, **kwargs)
        except TooManyRedirects:
            raise UserInfoException(self.code, '用户信息接口已关闭')
        except RequestException:
            raise UserInfoException(self.code, '获取用户信息失败')

        return UserlInfoParse(self.code, self.user.user_type, res.content).user_info


class UserlInfoParse():
    ''' 信息页面解析模块 '''

    def __init__(self, code, user_type, html):
        self.data = {}
        self.code = code
        coding = 'gbk' if user_type else 'GB18030'
        self.soup = BeautifulSoup(html.decode(coding), "html.parser")
        [self._html_parse_of_student, self._html_parse_of_teacher][user_type]()

    def _html_parse_of_student(self):
        table = self.soup.find("table", {"class": "formlist"})
        if not table:
            raise UserInfoException(self.code, '获取学生用户信息失败')

        real_name = table.find(id="xm").text
        grade = table.find(id="lbl_dqszj").text
        class_name = table.find(id="lbl_xzb").text
        faculty = table.find(id="lbl_xy").text
        specialty = table.find(id="lbl_zymc").text
        enrol_time = table.find(id="lbl_rxrq").text
        education_level = table.find(id="lbl_CC").text
        eductional_systme = table.find(id="lbl_xz").text

        sfzh = table.find(id="lbl_sfzh")
        id_card = sfzh.text if sfzh else table.find(id="sfzh")['value']

        csrq = table.find(id="lbl_csrq")
        birth_date = csrq.text if csrq else table.find(id="csrq")['value']

        lydq = table.find(id="lbl_lydq")
        hometown = lydq.text if lydq else table.find(id="lydq")['value']

        xb = table.find(id='XB')
        sex = xb.find('option', attrs={'selected': 'selected'}).text if xb else table.find(id="lbl_xb").text

        self.data = {
            "real_name": real_name,
            "sex": sex,
            "grade": grade,
            "birth_date": None if birth_date == 'NULL' else birth_date.replace('/', '-'),
            "class_name": class_name,
            "faculty": faculty,
            "specialty": specialty,
            "hometown": hometown,
            "enrol_time": enrol_time.replace('/', '-'),
            "education_level": education_level,
            "eductional_systme": eductional_systme,
            "id_card": id_card
        }

    def _html_parse_of_teacher(self):
        table = self.soup.find(id="Table3")
        if not table:
            raise UserInfoException(self.code, '获取教师用户信息失败')

        real_name = table.find(id='xm').text
        sex = table.find(id='xb').text
        dept = table.find(id='bm').text
        position = table.find(id='zw').text
        associate_degree = table.find(id='xl').text
        positional_title = table.find(id='zc').text
        self.data = {
            "real_name": real_name,
            "sex": sex,
            "dept": dept,
            "position": position,
            "associate_degree": associate_degree,
            "positional_title": positional_title
        }

    @property
    def user_info(self):
        ''' 返回用户信息json格式 '''
        return self.data
